#!/usr/bin/perl

############################################################################################
#      costfunction.pl [.ncd or .v file]
#	 If given a .ncd file, the script will run netgen and perform cost analysis on its
#      output. If given just a verilog file, the script will simply perform cost analysis
#      on the verilog file. 
############################################################################################\

use strict;

my @instancecosts =	(	857,
					1713,
					107,
					107,
					13,
					857,
					1716,
					54,
					377475,
					188738,
					302000,
					0,
					0,
					0,
					0,
					0,
					188738,
					188738,
					377475,
					377475
				);

my @instancenames = 	(	'X_LUT5',
					'X_LUT6',
					'X_SFF',
					'X_FF',
					'X_MUX2',
					'X_RAMD32',
					'X_RAMD64',
					'X_CARRY4',
					'X_RAMB36',
					'X_RAMB18',
					'X_DSP48E',
					'X_CKBUF',
					'X_BUF',
					'X_OBUF',
					'X_ONE',
					'X_ZERO',
					'X_FIFO18',
					'X_FIFO18_36',
					'X_FIFO36',
					'X_FIFO36_72',
				);

my $bramname = 		'X_RAMB';
my $bififoname=          'X_FIFO';

#Argument check
if (@ARGV != 1 || (!($ARGV[0] =~ /.*\.ncd$/) && !($ARGV[0] =~ /.*\.v$/)) )
{
      print "USAGE: costfunction.pl [.ncd or .v file]\n";
	print "If given a .ncd file, the script will run netgen and perform cost analysis on its" .
      	"output. If given just a verilog file, the script will perform cost analysis" .
      	"on the verilog file. The verilog file needs to be the output of a previous netgen" .
		"operation.";
      exit;
}

my @instancecounts;
my @totalcosts;

my $totalcost = 0;
my $totalbramcost = 0;
my $ncdfile = $ARGV[0];
my $inputfile = $ARGV[0] . '_ecn.v';

if ($ARGV[0] =~ /.*\.v$/)
{
	$inputfile = $ARGV[0];
}
else
{
	print "Running netgen ...\n";
	print `netgen -w -sim -ofmt verilog $ncdfile $inputfile`;
}

print "Running cost analysis ...\n";

open(INPUT, '<', $inputfile) or die "Could not open $inputfile for reading!";

#Read from data file
my @inputdata = <INPUT>;

for (my $i = 0; $i < @instancenames; $i++)
{
	$instancecounts[$i] = 0;
	$totalcosts[$i] = 0;
}

# Count how many of each instance is in the verilog

foreach my $line(@inputdata)
{
	chomp($line);
	if ($line =~ /\s*(X_\w*)\s/)
	{
		my $matched = 0;
		for (my $i = 0; $i < @instancenames; $i++)
		{
			if ($line =~ /$instancenames[$i]/) 
			{
				$matched = 1;
				$instancecounts[$i]++;
			}
		}

		if ($matched == 0)
		{
			push @instancenames, $1;
			push @instancecounts, 1;
			push @instancecosts, 0;
			push @totalcosts, 0;
		}
	}
}

# Compile total costs
for (my $i = 0; $i < @instancenames; $i++)
{
	$totalcosts[$i] = $instancecosts[$i] * $instancecounts[$i];
	$totalcost = $totalcost + $totalcosts[$i];
	if ($instancenames[$i] =~ /^$bramname/)
	{
		$totalbramcost = $totalbramcost + $totalcosts[$i];
	}
	if ($instancenames[$i] =~ /^$bififoname/)
	{
		$totalbramcost = $totalbramcost + $totalcosts[$i];
	}
}

# Instance counts printing
printf "\nCounts of all types of instances:\n";
for (my $i = 0; $i < @instancenames; $i++)
{
	printf "Number of %8s: %5d (Cost: %8d um^2 each, %10d um^2 total) \n", $instancenames[$i], 
		$instancecounts[$i], $instancecosts[$i], $totalcosts[$i];
}

printf "Total Cost                  : %f mm^2\n", $totalcost / 1000000.0;
printf "Total Cost (without BRAMs)  : %f mm^2\n", ($totalcost - $totalbramcost) / 1000000.0;

exit;
